import React, {Profiler} from "react";
import Leaflet from "./../maps/Leaflet";
import {measureTime} from './../helpers/measureTime';
import {CustomInput} from 'reactstrap';

export default class Leaflets extends React.Component{
  constructor(props) {
    super(props)
    this.metricsRef = React.createRef()
    this.metricsRef2 = React.createRef()
    this.state = {
      markers: [],
      markersFromTo: [],
      noOfMaps: 1,
      noOfMarkers: 10,
      noOfSteps: 500,
      currStep: 0,
      moveMarkers: false
    }
  }

  componentDidMount() {
    this.createRandomMarkers();
    this.setState({startTime: performance.now()});
    if(this.state.moveMarkers){
      this.timer = setInterval(() => {
        this.setState({currStep: this.state.currStep + 1});
        this.calcMarkerPosition()
      }, 10);
    }
  }

  componentDidUpdate() {
    if(this.state.currStep===this.state.noOfSteps){
      let endTime = performance.now();
      let duration = endTime - this.state.startTime
      let msg = "Duration " + duration.toFixed(3) + " for " + this.state.noOfSteps + " steps"
      console.log(msg)
      this.metricsRef2.current.innerHTML = msg
      this.setState({currStep: this.state.currStep + 1});
    }
  }

  componentWillUnmount() {
    clearInterval(this.timer);
  }

  createRandomMarkers = () => {
    let latStart = 4833125
    let lngStart = 1430784
    let markersFromTo = []
    let markers = []
    for (let i=0; i<this.state.noOfMarkers; i++) {
      let latRandom = Math.floor((Math.random() * 1063))
      let latFrom = (latStart + latRandom) / 100000
      latRandom = Math.floor((Math.random() * 1063))
      let latTo = (latStart + latRandom) / 100000
      let lngRandom = Math.floor((Math.random() * 2533))
      let lngFrom = (lngStart + lngRandom) / 100000
      lngRandom = Math.floor((Math.random() * 2533))
      let lngTo = (lngStart + lngRandom) / 100000
      markersFromTo.push({from: [latFrom, lngFrom], to: [latTo, lngTo]})
      markers.push([latFrom, lngFrom])
    }
    this.setState({markersFromTo: markersFromTo})
    this.setState({markers: markers})
  }

  calcMarkerPosition = () => {
    let noOfSteps = this.state.noOfSteps
    let currStep = this.state.currStep
    if (parseInt(currStep/noOfSteps)%2 === 0){
      currStep = currStep % noOfSteps
    } else{
      currStep = noOfSteps - (currStep % noOfSteps)
    }
    let markers = []
    for (let marker of this.state.markersFromTo){
      let latStart = marker.from[0]
      let lngStart = marker.from[1]
      let latEnd = marker.to[0]
      let lngEnd = marker.to[1]
//      markers.push([latStart, lngStart])
//      markers.push([latEnd, lngEnd])
      let latDiff = latEnd - latStart
      let lngDiff = lngEnd - lngStart
      let lat = latDiff / noOfSteps * currStep + latStart
      let lng = lngDiff / noOfSteps * currStep + lngStart
      markers.push([lat, lng])
    }
    this.setState({markers: markers})
  }

  changeSwitch = (event) => {
    if(this.state.moveMarkers){
      clearInterval(this.timer);
    }else{
      this.timer = setInterval(() => {
        this.setState({currStep: this.state.currStep + 1});
        this.calcMarkerPosition()
      }, 10);
    }
    this.setState({moveMarkers: !this.state.moveMarkers})
  }

  render(){
    return (
      <div className="mapTestContainer">
        <div className="mapTime">
          <CustomInput type="switch" onChange={this.changeSwitch} id="exampleCustomSwitch" name="customSwitch" label="Move markers" />
          <div ref={this.metricsRef2}>
            <span>{this.state.currStep} steps</span>
          </div>
          <span>Number of Markers: {this.state.noOfMarkers}</span>
          <div ref={this.metricsRef}>
          </div>
        </div>
        <div className="mapTest">
          <Profiler id="leaflet" onRender={measureTime(this)}>
            <Leaflet markers={this.state.markers}/>
          </Profiler>
        </div>
      </div>
    );
  }
}